@@ -191,7 +191,9 @@ adjust.to.dataset <- function(country, q, adj.dataset=NULL, adj.file=NULL, years
 		med <- apply(q, 1, 'median')[colnames(adj.dataset[,colidx])]
 		dif <- as.matrix(med - adj.dataset[idx1,colidx])
 		res <- aaply(q[colnames(adj.dataset[,colidx]),], 2, '-', dif)
-		res <- rbind(q[1,], aperm(res, c(2,1)))
+		res <- aperm(res, c(2,1))
+		if(! rownames(q)[1] %in% rownames(res))
+			res <- rbind(q[1,], res) # add current year
 		rownames(res) <- rownames(q)
 		return(res)
 	}
