@@ -439,57 +439,27 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, fi
 	MXf <- MXf[,c('country_code', 'age', estim.periods)]
 	
 	estim.years <- cols.starty[start.index:present.index] + 3
-	# Get sex ratio at birth
-	if(is.null(inputs$srb)) 
-		SRB <- load.wpp.dataset('sexRatio', wpp.year)
-	else SRB <- read.pop.file(inputs$srb)
-	names.SRB.data <- names(SRB)
-	num.columns <- grep('^[0-9]{4}.[0-9]{4}$', names.SRB.data) # index of year-columns
-	cols.starty <- as.integer(substr(names.SRB.data[num.columns], 1,4))
-    cols.endy <- as.integer(substr(names.SRB.data[num.columns], 6,9))
-	start.index <- which((cols.starty <= present.year) & (cols.endy > present.year))
-	end.index <- which((cols.endy >= end.year) & (cols.starty < end.year))
-	if(length(end.index) == 0) {
-		end.index <- length(num.columns)
-		warning('Data for SexRatioAtBirth not available for all projection periods.\nLast projection period set to ', 
-					names.SRB.data[num.columns[end.index]])
-	}
-	proj.periods <- names.SRB.data[num.columns[start.index:end.index]]
-	obs.periods <- NULL
-	if(start.index > 1) {
-		obs.periods <- names.SRB.data[num.columns[1:(start.index-1)]]
-		observed$SRB <- SRB[,c('country_code', obs.periods)]
-	}
-	SRB <- SRB[,c('country_code', proj.periods)]
-	proj.years <- cols.starty[start.index:end.index] + 3
-	
-	# Get percentage age-specific fertility rate
-	if(is.null(inputs$pasfr)) 
-		PASFR <- load.wpp.dataset('percentASFR', wpp.year)
-	else PASFR <- read.pop.file(inputs$pasfr)
-	if(!is.null(obs.periods)) {
-		avail.obs.periods <- is.element(obs.periods, colnames(PASFR))
-		observed$PASFR <- PASFR[,c('country_code', 'age', obs.periods[avail.obs.periods])]
-	}
-	PASFR <- PASFR[,c('country_code', 'age', proj.periods)]
-	
-	# Get migration type and base year
-	if(is.null(inputs$mig.type)) 
-		vwBase <- read.bayesPop.file(paste('vwBaseYear', wpp.year, '.txt', sep=''))
-	else vwBase <- read.pop.file(inputs$mig.type)
-	MIGtype <- vwBase[,c('country_code', 'ProjFirstYear', 'MigCode')]
 
-	create.pattern <- function(dataset, columns) {
-		pattern <- data.frame(dataset[,'country_code'])
-		for(col in columns)
-			if(col %in% colnames(dataset))
-				pattern <- cbind(pattern, dataset[,col])
-		if(ncol(pattern)==1) pattern <- NULL
-		else colnames(pattern) <- c('country_code', columns)[1:ncol(pattern)]
-		return(pattern)
-	}
-	MXpattern <- create.pattern(vwBase, c("AgeMortalityType", "AgeMortalityPattern", "LatestAgeMortalityPattern", "SmoothLatestAgeMortalityPattern", "WPPAIDS"))
-	PASFRpattern <- create.pattern(vwBase, c("PasfrNorm", paste0("Pasfr", .remove.all.spaces(levels(vwBase$PasfrNorm)))))
+	# Get sex ratio at birth
+	srblist <- .get.srb.data.and.time.periods(inputs$srb, present.year, end.year, wpp.year)
+	SRB <- srblist$srb
+	observed$SRB <- srblist$obs.srb
+	proj.periods <- srblist$proj.periods
+	obs.periods <- srblist$obs.periods
+	proj.years <- srblist$proj.years
+
+	# Get percentage age-specific fertility rate
+	pasfrlist <- .get.pasfr.data(inputs$pasfr, wpp.year, obs.periods, proj.periods)
+	PASFR <- pasfrlist$pasfr
+	observed$PASFR <- pasfrlist$obs.pasfr
+	
+	# Get migration type, migration base year, mx & pasfr patterns
+	patterns <- .get.mig.mx.pasfr.patterns(inputs, wpp.year)
+	MIGtype <- patterns$mig.type
+	MXpattern <- patterns$mx.pattern
+	PASFRpattern <- patterns$pasfr.pattern
+	
+
 	# Get age-specific migration
 	wppds <- data(package=paste0('wpp', wpp.year))
 	recon.mig <- NULL
@@ -612,26 +582,10 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, fi
 		} else
 			e0Mpred <- .load.wpp.traj('e0M', wpp.year)
 	}
-	}
+	} # end if(!fixed.mx)
+	
 	# Get TFR
-	if(!is.null(inputs$tfr.file)) {
-		if(inputs$tfr.file == 'median_')
-			TFRpred <- .load.wpp.traj('tfr', wpp.year, median.only=TRUE)
-		else {
-			file.name <- inputs$tfr.file
-			if(!file.exists(file.name))
-				stop('File ', file.name, 
-					' does not exist.\nSet tfr.sim.dir, tfr.file or change WPP year.')
-			if(verbose) cat('\nLoading ', file.name, '\n')
-			TFRpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
-			TFRpred <- TFRpred[,c('LocID', 'Year', 'Trajectory', 'TF')]
-			colnames(TFRpred) <- c('country_code', 'year', 'trajectory', 'value')
-		} 
-	} else {
-		if(!is.null(inputs$tfr.sim.dir)) 
-			TFRpred <- get.tfr.prediction(inputs$tfr.sim.dir, mcmc.dir=NA)
-		else TFRpred <- .load.wpp.traj('tfr', wpp.year)
-	}
+	TFRpred <- .get.tfr.data(inputs, wpp.year, verbose=verbose)
 	
 	inp <- new.env()
 	for(par in c('POPm0', 'POPf0', 'MXm', 'MXf', 'MXm.pred', 'MXf.pred', 'MXpattern', 'SRB',
@@ -644,6 +598,90 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, fi
 	return(inp)
 }
 
+.get.srb.data.and.time.periods <- function(srb.file, present.year, end.year, wpp.year) {
+    if(is.null(srb.file)) 
+        SRB <- load.wpp.dataset('sexRatio', wpp.year)
+    else SRB <- read.pop.file(srb.file)
+    names.SRB.data <- names(SRB)
+    num.columns <- grep('^[0-9]{4}.[0-9]{4}$', names.SRB.data) # index of year-columns
+    cols.starty <- as.integer(substr(names.SRB.data[num.columns], 1,4))
+    cols.endy <- as.integer(substr(names.SRB.data[num.columns], 6,9))
+    start.index <- which((cols.starty <= present.year) & (cols.endy > present.year))
+    end.index <- which((cols.endy >= end.year) & (cols.starty < end.year))
+    if(length(end.index) == 0) {
+        end.index <- length(num.columns)
+        warning('Data for SexRatioAtBirth not available for all projection periods.\nLast projection period set to ', 
+                names.SRB.data[num.columns[end.index]])
+    }
+    proj.periods <- names.SRB.data[num.columns[start.index:end.index]]
+    obs.periods <- NULL
+    obs.SRB <- NULL
+    if(start.index > 1) {
+        obs.periods <- names.SRB.data[num.columns[1:(start.index-1)]]
+        obs.SRB <- SRB[,c('country_code', obs.periods)]
+    }
+    SRB <- SRB[,c('country_code', proj.periods)]
+    proj.years <- cols.starty[start.index:end.index] + 3
+    return(list(srb=SRB, obs.srb=obs.SRB, proj.periods=proj.periods, 
+           obs.periods=obs.periods, proj.years=proj.years))
+}
+
+.get.pasfr.data <- function(pasfr.file, wpp.year, obs.periods, proj.periods) {
+    if(is.null(pasfr.file)) 
+        PASFR <- load.wpp.dataset('percentASFR', wpp.year)
+    else PASFR <- read.pop.file(pasfr.file)
+    obs.PASFR <- NULL
+    if(!is.null(obs.periods)) {
+        avail.obs.periods <- is.element(obs.periods, colnames(PASFR))
+        obs.PASFR <- PASFR[,c('country_code', 'age', obs.periods[avail.obs.periods])]
+    }
+    PASFR <- PASFR[,c('country_code', 'age', proj.periods)]
+    return(list(pasfr=PASFR, obs.pasfr=obs.PASFR))
+}
+
+.get.mig.mx.pasfr.patterns <- function(inputs, wpp.year) {
+    pattern.file <- if(!is.null(inputs$patterns)) inputs$patterns else inputs$mig.type
+    if(is.null(pattern.file)) 
+        vwBase <- read.bayesPop.file(paste('vwBaseYear', wpp.year, '.txt', sep=''))
+    else vwBase <- read.pop.file(pattern.file)
+    MIGtype <- vwBase[,c('country_code', 'ProjFirstYear', 'MigCode')]
+    
+    create.pattern <- function(dataset, columns) {
+        pattern <- data.frame(dataset[,'country_code'])
+        for(col in columns)
+            if(col %in% colnames(dataset))
+                pattern <- cbind(pattern, dataset[,col])
+        if(ncol(pattern)==1) pattern <- NULL
+        else colnames(pattern) <- c('country_code', columns)[1:ncol(pattern)]
+        return(pattern)
+    }
+    MXpattern <- create.pattern(vwBase, c("AgeMortalityType", "AgeMortalityPattern", "LatestAgeMortalityPattern", "SmoothLatestAgeMortalityPattern", "WPPAIDS"))
+    PASFRpattern <- create.pattern(vwBase, c("PasfrNorm", paste0("Pasfr", .remove.all.spaces(levels(vwBase$PasfrNorm)))))
+    return(list(mig.type=MIGtype, mx.pattern=MXpattern, pasfr.pattern=PASFRpattern))
+}
+
+.get.tfr.data <- function(inputs,  wpp.year, verbose=FALSE) {
+  if(!is.null(inputs$tfr.file)) {
+    if(inputs$tfr.file == 'median_')
+      TFRpred <- .load.wpp.traj('tfr', wpp.year, median.only=TRUE)
+    else {
+      file.name <- inputs$tfr.file
+      if(!file.exists(file.name))
+        stop('File ', file.name, 
+             ' does not exist.\nSet tfr.sim.dir, tfr.file or change WPP year.')
+      if(verbose) cat('\nLoading ', file.name, '\n')
+      TFRpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
+      TFRpred <- TFRpred[,c('LocID', 'Year', 'Trajectory', 'TF')]
+      colnames(TFRpred) <- c('country_code', 'year', 'trajectory', 'value')
+    } 
+  } else {
+    if(!is.null(inputs$tfr.sim.dir)) 
+      TFRpred <- get.tfr.prediction(inputs$tfr.sim.dir, mcmc.dir=NA)
+    else TFRpred <- .load.wpp.traj('tfr', wpp.year)
+  }
+    return(TFRpred)
+}
+
 .set.inp.migration.if.needed <- function(inputs, inpc, country) {
 	# If the age-specific migration values in "inputs" are NA 
 	# (e.g. because it was reconstructed later in the code), 
@@ -741,6 +779,7 @@ compute.pasfr.global.norms <- function(inputs) {
 	for(norm in norms) {
 		tpasfr <- NULL
 		countries <- pattern$country_code[which(pattern[[norm]]==1)]
+		if(length(countries) == 0) next
 		for(country in countries) {
 			pasfr <- .get.par.from.inputs('PASFR', inputs$observed, country)
 			tpasfr <- if(is.null(tpasfr)) pasfr else tpasfr + pasfr
@@ -1708,7 +1747,7 @@ LifeTableMxCol <- function(mx, colname=c('Lx', 'lx', 'qx', 'mx', 'dx', 'Tx', 'sx
 }
 
 
-LifeTableMx <- function(mx, sex=c('Male', 'Female'), include01=TRUE){
+LifeTableMx <- function(mx, sex=c('Male', 'Female'), include01=TRUE, radix = 1, open.age = 130){
 	# The first two elements of mx must correspond to 0-1 and 1-4. 
 	# If include01 is FALSE, the first two age groups of the results are collapsed to 0-5
 	sex <- match.arg(sex)
@@ -1717,6 +1756,7 @@ LifeTableMx <- function(mx, sex=c('Male', 'Female'), include01=TRUE){
 	Lx <- lx <- qx <- Tx <- sx <- dx <- ax <- rep(0, nage)
 	nagem1 <- nage-1
 	resage <- c(if(include01) c(0,1) else 0, seq(5, by=5, length=nage-2))
+	resage <- resage[resage <= open.age]
 	nresage <- length(resage)
 	nas <- rep(NA,nresage)
 	if(any(is.na(mx))) # there are NAs in mx
@@ -1729,11 +1769,34 @@ LifeTableMx <- function(mx, sex=c('Male', 'Female'), include01=TRUE){
 	LT$ax[nage] <- LT$ex[nage]
 	if(!include01) { # collapse 0-1 and 1-5 into 0-5
 		age05 <- c(FALSE, FALSE, TRUE)
-		LTres <- data.frame(age=resage)
+		LTres <- data.frame(age=seq(0, by=5, length=nage-1))
 		for(colname in colnames(LT)[-1])
 			LTres[[colname]] <- do.call(paste('.collapse', colname, sep='.'), list(LT, age05=age05))
 		LT <- LTres
 	}
+	if(radix != 1) 
+		LT <- transform(LT, lx = lx * radix, 
+						dx = dx * radix,
+						Lx = Lx * radix,
+						Tx = Tx * radix
+					)
+	if(open.age < 130) {
+		## truncate life table with open age group < 130+ (Patrick Gerland's code)
+		LT <- LT[1:nresage,]
+		## mx for open age /* reciprocal of 1/ex for open age group*/ 
+		LT$mx[nresage] <- 1 / LT$ex[nresage]
+		LT$qx[nresage] <- NA
+		LT$dx[nresage] <- LT$lx[nresage] ## open age group dx = lx
+		LT$Lx[nresage] <- LT$Tx[nresage] ## open age group Lx = Tx
+		# Sx
+        # for open age group, e.g., 85+ 
+        ## penultimate age group -> Last entry of S(x,n) is S( 80+,5) = T( 85) / T( 80)
+        ## for open age group itself: Sx cannot be computed due to trunaction Sx <- NA
+        LT$sx[nresage-1] <- LT$Tx[nresage]/LT$Tx[nresage-1]
+        LT$sx[nresage] <- NA
+        LT$ax[nresage] <- LT$ex[nresage] ## for open age group ax = ex
+	} 
+	rownames(LT) <- c(paste(LT$age[-nresage], pmax(LT$age[-1]-1,1), sep="-"), paste0(LT$age[nresage], "+"))
 	return(LT)
 }
 
@@ -1799,8 +1862,11 @@ age.specific.migration <- function(wpp.year=2015, years=seq(1955, 2100, by=5), c
 	if(is.null(countries)) {
 		countries <- mig$country_code
 		# filter out non-countries
-		locs <- bayesTFR:::load.bdem.dataset('UNlocations', wpp.year, envir=globalenv())
-		countries <- countries[countries %in% locs[locs$location_type==4, "country_code"]]
+		if(!exists("UNlocations")) 
+            bayesTFR:::load.bdem.dataset('UNlocations', wpp.year, envir=globalenv())
+		locs <- UNcountries()
+		#locs <- bayesTFR:::load.bdem.dataset('UNlocations', wpp.year, envir=globalenv())
+		countries <- countries[countries %in% locs]
 	} else mig <- mig[which(mig$country_code %in% countries),]
 	depratio.correction <- FALSE
 	if (depratio == TRUE || is.character(depratio)) {
